/*
 * store.h
 * Copyright (C) Martin Singer 2009-2011 <m_power3@users.sourceforge.net>
 * 
 * pdfchain is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pdfchain is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PDFCHAIN_WINDOW_STORE_H
#define PDFCHAIN_WINDOW_STORE_H

#include "pdfchain.h"


/*** Tree Model Column Record : Selection *************************************/

class
cTMCRecord_Selection : public Gtk::TreeModel::ColumnRecord
{
	public:

		cTMCRecord_Selection() {
			add( mTMColumn_ID		);
			add( mTMColumn_Label	);
			add( mTMColumn_Command	);
		}

		virtual ~cTMCRecord_Selection() {
			std::cout << std::endl << "cTMCRecord_Selection::~cTMCRecord_Selection()";	//TEST
		}
		
		Gtk::TreeModelColumn<guchar>		mTMColumn_ID;
		Gtk::TreeModelColumn<Glib::ustring>	mTMColumn_Label;
		Gtk::TreeModelColumn<std::string>	mTMColumn_Command;
};



/*** ListStore : Output ID ****************************************************/

class
cLStore_OutputID : public Gtk::ListStore
{
	public:

		cLStore_OutputID()
		:
			pTMCRecord( NULL )
		{
			pTMCRecord	= new cTMCRecord_Selection();
			set_column_types( *pTMCRecord );
		}

		virtual ~cLStore_OutputID() {
			std::cout << std::endl << "cLStore_OutputID::~cLStore_OutputID()";	//TEST
			if ( pTMCRecord != NULL ) { delete pTMCRecord;	pTMCRecord = NULL; }
		}

		static Glib::RefPtr<cLStore_OutputID> create() {
			return Glib::RefPtr<cLStore_OutputID>( new cLStore_OutputID() );
		}

		cTMCRecord_Selection* getTMCRecord() {
			return pTMCRecord;
		}

		Gtk::TreeModel::iterator setMultipleFiles( guchar output_id ) {
			
			removeRows();
			
			Gtk::TreeModel::iterator iter;

			iter = append();
			(*iter)[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::ID_NEW;
			(*iter)[ pTMCRecord->mTMColumn_Label	]	= _("Create new ID");
			(*iter)[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::ID_NEW;

			iter = append();
			(*iter)[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::ID_FIRST;
			(*iter)[ pTMCRecord->mTMColumn_Label	]	= _("Keep ID of first PDF file");
			(*iter)[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::ID_FIRST;

			iter = append();
			(*iter)[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::ID_FINAL;
			(*iter)[ pTMCRecord->mTMColumn_Label	]	= _("Keep ID of final PDF file");
			(*iter)[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::ID_FINAL;

			iter = children().begin();
			while ( children().end() != iter ) {
				if ( (*iter)[pTMCRecord->mTMColumn_ID] == output_id )
					break;
				iter++;
			}

			if ( children().end() == iter )
				iter = children().begin();
			
			return iter;
		}

		Gtk::TreeModel::iterator setSingleFile() {

			removeRows();
			//setMultipleFiles();
			
			Gtk::TreeModel::iterator iter;

			iter = append();
			(*iter)[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::ID_KEEP;
			(*iter)[ pTMCRecord->mTMColumn_Label	]	= _("Keep ID");
			(*iter)[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::ID_KEEP;

			return iter;
		}
		
	protected:
		
		cTMCRecord_Selection* pTMCRecord;

		void removeRows() {
			Gtk::TreeModel::iterator iter = children().begin();
			while ( children().end() != iter )	erase( iter++ );
			return;
		}
};


/*** ListStore : Even Odd *****************************************************/

class
cLStore_EvenOdd : public Gtk::ListStore
{
	public:

		cLStore_EvenOdd()
		:
			pTMCRecord( NULL )
		{

			pTMCRecord	= new cTMCRecord_Selection();
			set_column_types( *pTMCRecord );

			Gtk::TreeModel::Row	row;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::PAGES_ALL;
			row[ pTMCRecord->mTMColumn_Label	]	= _("all");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::PAGES_ALL;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::PAGES_EVEN;
			row[ pTMCRecord->mTMColumn_Label	]	= _("even");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::PAGES_EVEN;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::PAGES_ODD;
			row[ pTMCRecord->mTMColumn_Label	]	= _("odd");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::PAGES_ODD;
		}

		virtual ~cLStore_EvenOdd() {
			std::cout << std::endl << "cLStore_EvenOdd::~cLStore_EvenOdd()";	//TEST
			if ( pTMCRecord != NULL ) { delete pTMCRecord;	pTMCRecord = NULL; }
		}

		static Glib::RefPtr<cLStore_EvenOdd> create() {
			return Glib::RefPtr<cLStore_EvenOdd>( new cLStore_EvenOdd() );
		}

		cTMCRecord_Selection* getTMCRecord() {
			return pTMCRecord;
		}

	protected:

		cTMCRecord_Selection* pTMCRecord;
};



/*** ListStore : Rotation *****************************************************/

class
cLStore_Rotation : public Gtk::ListStore
{
	public:

		cLStore_Rotation()
		:
			pTMCRecord( NULL )
		{

			pTMCRecord	= new cTMCRecord_Selection();
			set_column_types( *pTMCRecord );

			Gtk::TreeModel::Row	row;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::ROTATION_N;
			row[ pTMCRecord->mTMColumn_Label	]	= _("0°");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::ROTATION_N;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::ROTATION_E;
			row[ pTMCRecord->mTMColumn_Label	]	= _("90°");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::ROTATION_E;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::ROTATION_S;
			row[ pTMCRecord->mTMColumn_Label	]	= _("180°");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::ROTATION_S;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::ROTATION_W;
			row[ pTMCRecord->mTMColumn_Label	]	= _("270°");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::ROTATION_W;
		}

		virtual ~cLStore_Rotation() {
			std::cout << std::endl << "cLStore_Rotation::~cLStore_Rotation()";	//TEST
			if ( pTMCRecord != NULL ) { delete pTMCRecord;	pTMCRecord = NULL; }
		}

		static Glib::RefPtr<cLStore_Rotation> create() {
			return Glib::RefPtr<cLStore_Rotation>( new cLStore_Rotation() );
		}

		cTMCRecord_Selection* getTMCRecord() {
			return pTMCRecord;
		}

	protected:
		
		cTMCRecord_Selection* pTMCRecord;
};



/*** ListStore : Counting Methode *********************************************/

class
cLStore_CountingMethode : public Gtk::ListStore
{
	public:

		cLStore_CountingMethode()
		:
			pTMCRecord( NULL )
		{

			pTMCRecord	= new cTMCRecord_Selection();
			set_column_types( *pTMCRecord );

			Gtk::TreeModel::Row	row;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::COUNT_OCT;
			row[ pTMCRecord->mTMColumn_Label	]	= _("Octal");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::COUNT_OCT;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::COUNT_DEC;
			row[ pTMCRecord->mTMColumn_Label	]	= _("Decimal");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::COUNT_DEC;

			row = *( this->append() );
			row[ pTMCRecord->mTMColumn_ID		]	= Pdfchain::Id::COUNT_HEX;
			row[ pTMCRecord->mTMColumn_Label	]	= _("Hexadeciaml");
			row[ pTMCRecord->mTMColumn_Command	]	= Pdfchain::Cmd::COUNT_HEX;
		}

		virtual ~cLStore_CountingMethode() {
			std::cout << std::endl << "cLStore_CountingMethode::~cLStore_CountingMethode()";	//TEST
			if ( pTMCRecord != NULL ) { delete pTMCRecord;	pTMCRecord = NULL; }
		}

		static Glib::RefPtr<cLStore_CountingMethode> create() {
			return Glib::RefPtr<cLStore_CountingMethode>( new cLStore_CountingMethode() );
		}

		cTMCRecord_Selection* getTMCRecord() {
			return pTMCRecord;
		}

	protected:
		
		cTMCRecord_Selection* pTMCRecord;
};


#endif